'''
Extends the DefaultPerk to read incoming chat messages and define a key for
switching focus between history and composition areas.

@author: Brett Clippingdale
@organization: IBM Corporation
@copyright: Copyright (c) 2006
@license: Common Public License 1.0

All rights reserved. This program and the accompanying materials are made 
available under the terms of the Common Public License v1.0 which accompanies
this distribution, and is available at 
U{http://www.opensource.org/licenses/cpl1.0.php}
'''
# import useful modules for Perks
import Perk, Task
from POR import POR
from i18n import bind, _

# metadata describing this Perk
__uie__ = dict(kind='perk', tier='gaim', all_tiers=False)

# to support translation of strings in this Perk, uncomment the following line
# and provide the proper domain and path to your translation file
# _ = bind(domain, locale_dir)

class GaimPerkBasic(Perk.Perk):
  '''
  Example Perk template. Use this file as a way to get started writing your 
  own Tasks and creating your own keyboard bindings. The getDescription method
  should be changed to return a translated description of this Perk. The 
  description should include a statement of which Tiers this Perk applies to
  by default.
  '''
  def init(self):
    #self.registerEventTask(HandleFocusChange(self))
    self.registerTask(ReadIncomingMessage(tier=True))
    self.registerTask(ToggleChatPanel('chat panel toggle'))
    # get the Keyboard device and set CapsLock as a modifier
    kbd = self.getInputDevice('Keyboard')
    self.addInputModifiers(kbd, kbd.AEK_CAPS_LOCK)
    # register CapsLock-S as the key to trigger the input task
    self.registerCommand(kbd, 'chat panel toggle', False, 
                         [kbd.AEK_CAPS_LOCK, kbd.AEK_T])
                         
  def getDescription(self):
    return _('Applies to gaim by default.')

class ReadIncomingMessage(Task.CaretTask):
  '''
  Reads text inserted into a text control starting with a newline.
  Usually, a message received.
  '''
  def executeInserted(self, por, text, **kwargs):
    if text.startswith('\n'):
      self.stopNow()
      self.sayItem(text=text)

class ToggleChatPanel(Task.InputTask):
  '''
  Toggles focus between chat composition and chat history panels.
  '''
  def execute(self, **kwargs):
    # get the frame
    frame = self.getRootAcc()
    # get the page tab list
    tab_list = self.getAccFromPath(frame, 0,1)
    # do a sanity check to make sure we're refering to the correct component
    if self.hasAccRole('page tab list', tab_list):
      # get active conversation (i.e. the selected tab)
      tab = self.getFirstSelected(tab_list)
      history = self.getAccFromPath(tab, 0,0,0,0,0)
      compose = self.getAccFromPath(tab, 0,0,1,1,0,0,0)
      # is composition text box currently focused?
      if self.hasAccState('focused', compose):
        # if so, change focus to chat history panel
        self.setAccFocus(history)
        # stop current speech
        self.stopNow()
        # inhibit the next stop
        self.inhibitMayStop()
        # use _() to support internationalization of text
        self.saySection(text=_('History,'))
      else:
        # otherwise, change focus to chat compose panel
        self.setAccFocus(compose)
        self.stopNow()
        self.inhibitMayStop()
        self.saySection(text=_('Compose,'))
